@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,6 +111,42 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "acpi.h"
@@ -118,8 +154,8 @@
 #include "acdebug.h"
 #include "acnamesp.h"
 #include "acpredef.h"
+#include "acinterp.h"
 
-#ifdef ACPI_DEBUGGER
 
 #define _COMPONENT          ACPI_CA_DEBUGGER
         ACPI_MODULE_NAME    ("dbtest")
@@ -153,6 +189,14 @@ AcpiDbTestStringType (
     ACPI_NAMESPACE_NODE     *Node,
     UINT32                  ByteLength);
 
+static ACPI_STATUS
+AcpiDbTestPackageType (
+    ACPI_NAMESPACE_NODE     *Node);
+
+static ACPI_STATUS
+AcpiDbTestFieldUnitType (
+    ACPI_OPERAND_OBJECT     *ObjDesc);
+
 static ACPI_STATUS
 AcpiDbReadFromObject (
     ACPI_NAMESPACE_NODE     *Node,
@@ -196,13 +240,13 @@ static ACPI_DB_ARGUMENT_INFO    AcpiDbTestTypes [] =
  * used to read and write the various namespace objects. The point
  * is to force the AML interpreter do all of the work.
  */
-#define                     ACPI_DB_READ_METHOD     "\\_T98"
-#define                     ACPI_DB_WRITE_METHOD    "\\_T99"
+#define ACPI_DB_READ_METHOD     "\\_T98"
+#define ACPI_DB_WRITE_METHOD    "\\_T99"
 
 static ACPI_HANDLE          ReadHandle = NULL;
 static ACPI_HANDLE          WriteHandle = NULL;
 
-/* ASL Definitions of the debugger read/write control methods */
+/* ASL Definitions of the debugger read/write control methods. AML below. */
 
 #if 0
 DefinitionBlock ("ssdt.aml", "SSDT", 2, "Intel", "DEBUG", 0x00000001)
@@ -355,7 +399,7 @@ AcpiDbTestAllObjects (
     /* Walk the entire namespace, testing each supported named data object */
 
     (void) AcpiWalkNamespace (ACPI_TYPE_ANY, ACPI_ROOT_OBJECT,
-                ACPI_UINT32_MAX, AcpiDbTestOneObject, NULL, NULL, NULL);
+        ACPI_UINT32_MAX, AcpiDbTestOneObject, NULL, NULL, NULL);
 }
 
 
@@ -368,10 +412,8 @@ AcpiDbTestAllObjects (
  * RETURN:      Status
  *
  * DESCRIPTION: Test one namespace object. Supported types are Integer,
- *              String, Buffer, BufferField, and FieldUnit. All other object
- *              types are simply ignored.
- *
- *              Note: Support for Packages is not implemented.
+ *              String, Buffer, Package, BufferField, and FieldUnit.
+ *              All other object types are simply ignored.
  *
  ******************************************************************************/
 
@@ -384,7 +426,6 @@ AcpiDbTestOneObject (
 {
     ACPI_NAMESPACE_NODE     *Node;
     ACPI_OPERAND_OBJECT     *ObjDesc;
-    ACPI_OPERAND_OBJECT     *RegionObj;
     ACPI_OBJECT_TYPE        LocalType;
     UINT32                  BitLength = 0;
     UINT32                  ByteLength = 0;
@@ -421,20 +462,28 @@ AcpiDbTestOneObject (
         BitLength = ByteLength * 8;
         break;
 
+    case ACPI_TYPE_PACKAGE:
+
+        LocalType = ACPI_TYPE_PACKAGE;
+        break;
+
     case ACPI_TYPE_FIELD_UNIT:
-    case ACPI_TYPE_BUFFER_FIELD:
     case ACPI_TYPE_LOCAL_REGION_FIELD:
     case ACPI_TYPE_LOCAL_INDEX_FIELD:
     case ACPI_TYPE_LOCAL_BANK_FIELD:
 
+        LocalType = ACPI_TYPE_FIELD_UNIT;
+        break;
+
+    case ACPI_TYPE_BUFFER_FIELD:
+        /*
+         * The returned object will be a Buffer if the field length
+         * is larger than the size of an Integer (32 or 64 bits
+         * depending on the DSDT version).
+         */
         LocalType = ACPI_TYPE_INTEGER;
         if (ObjDesc)
         {
-            /*
-             * Returned object will be a Buffer if the field length
-             * is larger than the size of an Integer (32 or 64 bits
-             * depending on the DSDT version).
-             */
             BitLength = ObjDesc->CommonField.BitLength;
             ByteLength = ACPI_ROUND_BITS_UP_TO_BYTES (BitLength);
             if (BitLength > AcpiGbl_IntegerBitWidth)
@@ -444,9 +493,9 @@ AcpiDbTestOneObject (
         }
         break;
 
-    default:
+default:
 
-        /* Ignore all other types */
+        /* Ignore all non-data types - Methods, Devices, Scopes, etc. */
 
         return (AE_OK);
     }
@@ -455,43 +504,13 @@ AcpiDbTestOneObject (
 
     AcpiOsPrintf ("%14s: %4.4s",
         AcpiUtGetTypeName (Node->Type), Node->Name.Ascii);
+
     if (!ObjDesc)
     {
-        AcpiOsPrintf (" Ignoring, no attached object\n");
+        AcpiOsPrintf (" No attached sub-object, ignoring\n");
         return (AE_OK);
     }
 
-    /*
-     * Check for unsupported region types. Note: AcpiExec simulates
-     * access to SystemMemory, SystemIO, PCI_Config, and EC.
-     */
-    switch (Node->Type)
-    {
-    case ACPI_TYPE_LOCAL_REGION_FIELD:
-
-        RegionObj = ObjDesc->Field.RegionObj;
-        switch (RegionObj->Region.SpaceId)
-        {
-        case ACPI_ADR_SPACE_SYSTEM_MEMORY:
-        case ACPI_ADR_SPACE_SYSTEM_IO:
-        case ACPI_ADR_SPACE_PCI_CONFIG:
-        case ACPI_ADR_SPACE_EC:
-
-            break;
-
-        default:
-
-            AcpiOsPrintf ("      %s space is not supported [%4.4s]\n",
-                AcpiUtGetRegionName (RegionObj->Region.SpaceId),
-                RegionObj->Region.Node->Name.Ascii);
-            return (AE_OK);
-        }
-        break;
-
-    default:
-        break;
-    }
-
     /* At this point, we have resolved the object to one of the major types */
 
     switch (LocalType)
@@ -511,6 +530,16 @@ AcpiDbTestOneObject (
         Status = AcpiDbTestBufferType (Node, BitLength);
         break;
 
+    case ACPI_TYPE_PACKAGE:
+
+        Status = AcpiDbTestPackageType (Node);
+        break;
+
+    case ACPI_TYPE_FIELD_UNIT:
+
+        Status = AcpiDbTestFieldUnitType (ObjDesc);
+        break;
+
     default:
 
         AcpiOsPrintf (" Ignoring, type not implemented (%2.2X)",
@@ -518,17 +547,11 @@ AcpiDbTestOneObject (
         break;
     }
 
-    switch (Node->Type)
+    /* Exit on error, but don't abort the namespace walk */
+
+    if (ACPI_FAILURE (Status))
     {
-    case ACPI_TYPE_LOCAL_REGION_FIELD:
-
-        RegionObj = ObjDesc->Field.RegionObj;
-        AcpiOsPrintf (" (%s)",
-            AcpiUtGetRegionName (RegionObj->Region.SpaceId));
-        break;
-
-    default:
-        break;
+        Status = AE_OK;
     }
 
     AcpiOsPrintf ("\n");
@@ -580,7 +603,7 @@ AcpiDbTestIntegerType (
         return (Status);
     }
 
-    AcpiOsPrintf (" (%4.4X/%3.3X) %8.8X%8.8X",
+    AcpiOsPrintf (ACPI_DEBUG_LENGTH_FORMAT " %8.8X%8.8X",
         BitLength, ACPI_ROUND_BITS_UP_TO_BYTES (BitLength),
         ACPI_FORMAT_UINT64 (Temp1->Integer.Value));
 
@@ -589,7 +612,6 @@ AcpiDbTestIntegerType (
     {
         ValueToWrite = 0;
     }
-
     /* Write a new value */
 
     WriteValue.Type = ACPI_TYPE_INTEGER;
@@ -703,8 +725,8 @@ AcpiDbTestBufferType (
 
     /* Emit a few bytes of the buffer */
 
-    AcpiOsPrintf (" (%4.4X/%3.3X)", BitLength, Temp1->Buffer.Length);
-    for (i = 0; ((i < 4) && (i < ByteLength)); i++)
+    AcpiOsPrintf (ACPI_DEBUG_LENGTH_FORMAT, BitLength, Temp1->Buffer.Length);
+    for (i = 0; ((i < 8) && (i < ByteLength)); i++)
     {
         AcpiOsPrintf (" %2.2X", Temp1->Buffer.Pointer[i]);
     }
@@ -718,7 +740,7 @@ AcpiDbTestBufferType (
      * count is not an integral number of bytes. Zero out the
      * unused bits.
      */
-    ACPI_MEMSET (Buffer, BUFFER_FILL_VALUE, ByteLength);
+    memset (Buffer, BUFFER_FILL_VALUE, ByteLength);
     ExtraBits = BitLength % 8;
     if (ExtraBits)
     {
@@ -743,7 +765,7 @@ AcpiDbTestBufferType (
         goto Exit;
     }
 
-    if (ACPI_MEMCMP (Temp2->Buffer.Pointer, Buffer, ByteLength))
+    if (memcmp (Temp2->Buffer.Pointer, Buffer, ByteLength))
     {
         AcpiOsPrintf (" MISMATCH 2: New buffer value");
     }
@@ -767,7 +789,8 @@ AcpiDbTestBufferType (
         goto Exit;
     }
 
-    if (ACPI_MEMCMP (Temp1->Buffer.Pointer, Temp3->Buffer.Pointer, ByteLength))
+    if (memcmp (Temp1->Buffer.Pointer,
+            Temp3->Buffer.Pointer, ByteLength))
     {
         AcpiOsPrintf (" MISMATCH 3: While restoring original buffer");
     }
@@ -817,13 +840,13 @@ AcpiDbTestStringType (
         return (Status);
     }
 
-    AcpiOsPrintf (" (%4.4X/%3.3X) \"%s\"", (Temp1->String.Length * 8),
+    AcpiOsPrintf (ACPI_DEBUG_LENGTH_FORMAT " \"%s\"", (Temp1->String.Length * 8),
         Temp1->String.Length, Temp1->String.Pointer);
 
     /* Write a new value */
 
     WriteValue.Type = ACPI_TYPE_STRING;
-    WriteValue.String.Length = ACPI_STRLEN (ValueToWrite);
+    WriteValue.String.Length = strlen (ValueToWrite);
     WriteValue.String.Pointer = ValueToWrite;
 
     Status = AcpiDbWriteToObject (Node, &WriteValue);
@@ -840,7 +863,7 @@ AcpiDbTestStringType (
         goto Exit;
     }
 
-    if (ACPI_STRCMP (Temp2->String.Pointer, ValueToWrite))
+    if (strcmp (Temp2->String.Pointer, ValueToWrite))
     {
         AcpiOsPrintf (" MISMATCH 2: %s, expecting %s",
             Temp2->String.Pointer, ValueToWrite);
@@ -848,7 +871,7 @@ AcpiDbTestStringType (
 
     /* Write back the original value */
 
-    WriteValue.String.Length = ACPI_STRLEN (Temp1->String.Pointer);
+    WriteValue.String.Length = strlen (Temp1->String.Pointer);
     WriteValue.String.Pointer = Temp1->String.Pointer;
 
     Status = AcpiDbWriteToObject (Node, &WriteValue);
@@ -865,7 +888,7 @@ AcpiDbTestStringType (
         goto Exit;
     }
 
-    if (ACPI_STRCMP (Temp1->String.Pointer, Temp3->String.Pointer))
+    if (strcmp (Temp1->String.Pointer, Temp3->String.Pointer))
     {
         AcpiOsPrintf (" MISMATCH 3: %s, expecting %s",
             Temp3->String.Pointer, Temp1->String.Pointer);
@@ -879,6 +902,107 @@ Exit:
 }
 
 
+/*******************************************************************************
+ *
+ * FUNCTION:    AcpiDbTestPackageType
+ *
+ * PARAMETERS:  Node                - Parent NS node for the object
+ *
+ * RETURN:      Status
+ *
+ * DESCRIPTION: Test read for a Package object.
+ *
+ ******************************************************************************/
+
+static ACPI_STATUS
+AcpiDbTestPackageType (
+    ACPI_NAMESPACE_NODE     *Node)
+{
+    ACPI_OBJECT             *Temp1 = NULL;
+    ACPI_STATUS             Status;
+
+
+    /* Read the original value */
+
+    Status = AcpiDbReadFromObject (Node, ACPI_TYPE_PACKAGE, &Temp1);
+    if (ACPI_FAILURE (Status))
+    {
+        return (Status);
+    }
+
+    AcpiOsPrintf (" %.2X Elements", Temp1->Package.Count);
+    AcpiOsFree (Temp1);
+    return (Status);
+}
+
+
+/*******************************************************************************
+ *
+ * FUNCTION:    AcpiDbTestFieldUnitType
+ *
+ * PARAMETERS:  ObjDesc                 - A field unit object
+ *
+ * RETURN:      Status
+ *
+ * DESCRIPTION: Test read/write on a named field unit.
+ *
+ ******************************************************************************/
+
+static ACPI_STATUS
+AcpiDbTestFieldUnitType (
+    ACPI_OPERAND_OBJECT     *ObjDesc)
+{
+    ACPI_OPERAND_OBJECT     *RegionObj;
+    UINT32                  BitLength = 0;
+    UINT32                  ByteLength = 0;
+    ACPI_STATUS             Status = AE_OK;
+    ACPI_OPERAND_OBJECT     *RetBufferDesc;
+
+
+    /* Supported spaces are memory/io/pci_config */
+
+    RegionObj = ObjDesc->Field.RegionObj;
+    switch (RegionObj->Region.SpaceId)
+    {
+    case ACPI_ADR_SPACE_SYSTEM_MEMORY:
+    case ACPI_ADR_SPACE_SYSTEM_IO:
+    case ACPI_ADR_SPACE_PCI_CONFIG:
+
+        /* Need the interpreter to execute */
+
+        AcpiUtAcquireMutex (ACPI_MTX_INTERPRETER);
+        AcpiUtAcquireMutex (ACPI_MTX_NAMESPACE);
+
+        /* Exercise read-then-write */
+
+        Status = AcpiExReadDataFromField (NULL, ObjDesc, &RetBufferDesc);
+        if (Status == AE_OK)
+        {
+            AcpiExWriteDataToField (RetBufferDesc, ObjDesc, NULL);
+            AcpiUtRemoveReference (RetBufferDesc);
+        }
+
+        AcpiUtReleaseMutex (ACPI_MTX_NAMESPACE);
+        AcpiUtReleaseMutex (ACPI_MTX_INTERPRETER);
+
+        BitLength = ObjDesc->CommonField.BitLength;
+        ByteLength = ACPI_ROUND_BITS_UP_TO_BYTES (BitLength);
+
+        AcpiOsPrintf (ACPI_DEBUG_LENGTH_FORMAT " [%s]", BitLength,
+            ByteLength, AcpiUtGetRegionName (RegionObj->Region.SpaceId));
+        return (Status);
+
+    default:
+
+        AcpiOsPrintf (
+            "      %s address space is not supported in this command [%4.4s]",
+            AcpiUtGetRegionName (RegionObj->Region.SpaceId),
+            RegionObj->Region.Node->Name.Ascii);
+        return (AE_OK);
+    }
+}
+
+
 /*******************************************************************************
  *
  * FUNCTION:    AcpiDbReadFromObject
@@ -919,9 +1043,10 @@ AcpiDbReadFromObject (
     ReturnObj.Length  = ACPI_ALLOCATE_BUFFER;
 
     AcpiGbl_MethodExecuting = TRUE;
-    Status = AcpiEvaluateObject (ReadHandle, NULL, &ParamObjects, &ReturnObj);
-    AcpiGbl_MethodExecuting = FALSE;
+    Status = AcpiEvaluateObject (ReadHandle, NULL,
+        &ParamObjects, &ReturnObj);
 
+    AcpiGbl_MethodExecuting = FALSE;
     if (ACPI_FAILURE (Status))
     {
         AcpiOsPrintf ("Could not read from object, %s",
@@ -936,6 +1061,7 @@ AcpiDbReadFromObject (
     case ACPI_TYPE_INTEGER:
     case ACPI_TYPE_BUFFER:
     case ACPI_TYPE_STRING:
+    case ACPI_TYPE_PACKAGE:
         /*
          * Did we receive the type we wanted? Most important for the
          * Integer/Buffer case (when a field is larger than an Integer,
@@ -947,6 +1073,7 @@ AcpiDbReadFromObject (
                 AcpiUtGetTypeName (ExpectedType),
                 AcpiUtGetTypeName (RetValue->Type));
 
+            AcpiOsFree (ReturnObj.Pointer);
             return (AE_TYPE);
         }
 
@@ -957,8 +1084,8 @@ AcpiDbReadFromObject (
 
         AcpiOsPrintf (" Unsupported return object type, %s",
             AcpiUtGetTypeName (RetValue->Type));
-        AcpiOsFree (ReturnObj.Pointer);
 
+        AcpiOsFree (ReturnObj.Pointer);
         return (AE_TYPE);
     }
 
@@ -998,7 +1125,7 @@ AcpiDbWriteToObject (
 
     /* Copy the incoming user parameter */
 
-    ACPI_MEMCPY (&Params[1], Value, sizeof (ACPI_OBJECT));
+    memcpy (&Params[1], Value, sizeof (ACPI_OBJECT));
 
     ParamObjects.Count = 2;
     ParamObjects.Pointer = Params;
@@ -1042,15 +1169,17 @@ AcpiDbEvaluateAllPredefinedNames (
 
     if (CountArg)
     {
-        Info.MaxCount = ACPI_STRTOUL (CountArg, NULL, 0);
+        Info.MaxCount = strtoul (CountArg, NULL, 0);
     }
 
     /* Search all nodes in namespace */
 
-    (void) AcpiWalkNamespace (ACPI_TYPE_ANY, ACPI_ROOT_OBJECT, ACPI_UINT32_MAX,
-                AcpiDbEvaluateOnePredefinedName, NULL, (void *) &Info, NULL);
+    (void) AcpiWalkNamespace (ACPI_TYPE_ANY, ACPI_ROOT_OBJECT,
+        ACPI_UINT32_MAX, AcpiDbEvaluateOnePredefinedName, NULL,
+        (void *) &Info, NULL);
 
-    AcpiOsPrintf ("Evaluated %u predefined names in the namespace\n", Info.Count);
+    AcpiOsPrintf (
+        "Evaluated %u predefined names in the namespace\n", Info.Count);
 }
 
 
@@ -1103,7 +1232,7 @@ AcpiDbEvaluateOnePredefinedName (
         return (AE_OK);
     }
 
-    Pathname = AcpiNsGetExternalPathname (Node);
+    Pathname = AcpiNsGetNormalizedPathname (Node, TRUE);
     if (!Pathname)
     {
         return (AE_OK);
@@ -1148,8 +1277,10 @@ AcpiDbEvaluateOnePredefinedName (
 
             case ACPI_TYPE_STRING:
 
-                ThisParam->String.Pointer = "This is the default argument string";
-                ThisParam->String.Length = ACPI_STRLEN (ThisParam->String.Pointer);
+                ThisParam->String.Pointer =
+                    "This is the default argument string";
+                ThisParam->String.Length =
+                    strlen (ThisParam->String.Pointer);
                 break;
 
             case ACPI_TYPE_BUFFER:
@@ -1188,7 +1319,8 @@ AcpiDbEvaluateOnePredefinedName (
 
     Status = AcpiEvaluateObject (Node, NULL, &ParamObjects, &ReturnObj);
 
-    AcpiOsPrintf ("%-32s returned %s\n", Pathname, AcpiFormatException (Status));
+    AcpiOsPrintf ("%-32s returned %s\n",
+        Pathname, AcpiFormatException (Status));
     AcpiGbl_MethodExecuting = FALSE;
     ACPI_FREE (Pathname);
 
@@ -1206,5 +1338,3 @@ AcpiDbEvaluateOnePredefinedName (
 
     return (Status);
 }
-
-#endif /* ACPI_DEBUGGER */
