@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2021, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -344,9 +344,7 @@ AcpiExLoadTableOp (
 
     /* Complete the initialization/resolution of new objects */
 
-    AcpiExExitInterpreter();
-    AcpiNsInitializeObjects();
-    AcpiExEnterInterpreter();
+    AcpiNsInitializeObjects ();
 
     /* Parameter Data (optional) */
 
