@@ -8,13 +8,13 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2012, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
  *
  * 2.1. This is your license from Intel Corp. under its intellectual property
- * rights.  You may have additional license terms from the party that provided
+ * rights. You may have additional license terms from the party that provided
  * you this software, covering your right to use that party's intellectual
  * property rights.
  *
@@ -31,7 +31,7 @@
  * offer to sell, and import the Covered Code and derivative works thereof
  * solely to the minimum extent necessary to exercise the above copyright
  * license, and in no event shall the patent license extend to any additions
- * to or modifications of the Original Intel Code.  No other license or right
+ * to or modifications of the Original Intel Code. No other license or right
  * is granted directly or by implication, estoppel or otherwise;
  *
  * The above copyright and patent license is granted only if the following
@@ -43,11 +43,11 @@
  * Redistribution of source code of any substantial portion of the Covered
  * Code or modification with rights to further distribute source must include
  * the above Copyright Notice, the above License, this list of Conditions,
- * and the following Disclaimer and Export Compliance provision.  In addition,
+ * and the following Disclaimer and Export Compliance provision. In addition,
  * Licensee must cause all Covered Code to which Licensee contributes to
  * contain a file documenting the changes Licensee made to create that Covered
- * Code and the date of any change.  Licensee must include in that file the
- * documentation of any changes made by any predecessor Licensee.  Licensee
+ * Code and the date of any change. Licensee must include in that file the
+ * documentation of any changes made by any predecessor Licensee. Licensee
  * must include a prominent statement that the modification is derived,
  * directly or indirectly, from Original Intel Code.
  *
@@ -55,7 +55,7 @@
  * Redistribution of source code of any substantial portion of the Covered
  * Code or modification without rights to further distribute source must
  * include the following Disclaimer and Export Compliance provision in the
- * documentation and/or other materials provided with distribution.  In
+ * documentation and/or other materials provided with distribution. In
  * addition, Licensee may not authorize further sublicense of source of any
  * portion of the Covered Code, and must include terms to the effect that the
  * license from Licensee to its licensee is limited to the intellectual
@@ -80,10 +80,10 @@
  * 4. Disclaimer and Export Compliance
  *
  * 4.1. INTEL MAKES NO WARRANTY OF ANY KIND REGARDING ANY SOFTWARE PROVIDED
- * HERE.  ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
- * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT,  ASSISTANCE,
- * INSTALLATION, TRAINING OR OTHER SERVICES.  INTEL WILL NOT PROVIDE ANY
- * UPDATES, ENHANCEMENTS OR EXTENSIONS.  INTEL SPECIFICALLY DISCLAIMS ANY
+ * HERE. ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
+ * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT, ASSISTANCE,
+ * INSTALLATION, TRAINING OR OTHER SERVICES. INTEL WILL NOT PROVIDE ANY
+ * UPDATES, ENHANCEMENTS OR EXTENSIONS. INTEL SPECIFICALLY DISCLAIMS ANY
  * IMPLIED WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT AND FITNESS FOR A
  * PARTICULAR PURPOSE.
  *
@@ -92,14 +92,14 @@
  * COSTS OF PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, OR FOR ANY INDIRECT,
  * SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS AGREEMENT, UNDER ANY
  * CAUSE OF ACTION OR THEORY OF LIABILITY, AND IRRESPECTIVE OF WHETHER INTEL
- * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES.  THESE LIMITATIONS
+ * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES. THESE LIMITATIONS
  * SHALL APPLY NOTWITHSTANDING THE FAILURE OF THE ESSENTIAL PURPOSE OF ANY
  * LIMITED REMEDY.
  *
  * 4.3. Licensee shall not export, either directly or indirectly, any of this
  * software or system incorporating such software without first obtaining any
  * required license or other approval from the U. S. Department of Commerce or
- * any other agency or department of the United States Government.  In the
+ * any other agency or department of the United States Government. In the
  * event Licensee exports any such software from the United States or
  * re-exports any such software from a foreign destination, Licensee shall
  * ensure that the distribution and export/re-export of the software is in
@@ -111,15 +111,52 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
 #include "acpihelp.h"
 
+
 /*
  * AML opcodes with related syntax and grammar information.
  * This table was extracted from the ACPI specification.
  */
-const AH_AML_OPCODE         AmlOpcodeInfo[] =
+const AH_AML_OPCODE         Gbl_AmlOpcodeInfo[] =
 {
     {0x00,   0x00,   "0x00",            "ZeroOp",               "DataObject",   NULL,                                           NULL,
                                                                 NULL},
@@ -157,7 +194,9 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
                                                                 "DefVarPackage := VarPackageOp PkgLength VarNumElements PackageElementList"},
     {0x14,   0x14,   "0x14",            "MethodOp",             "TermObject",   "NameString ByteData",                          "TermList",
                                                                 "DefMethod := MethodOp PkgLength NameString MethodFlags TermList"},
-    {0x15,   0x2D,   "0x15-0x2D",       NULL,                   NULL,           NULL,                                           NULL,
+    {0x15,   0x15,   "0x15",            "ExternalOp",           "NameObject",   "NameString ByteData ByteData",                 NULL,
+                                                                "DefExternal := ExternalOp NameString ObjectType ArgumentCount"},
+    {0x16,   0x2D,   "0x16-0x2D",       NULL,                   NULL,           NULL,                                           NULL,
                                                                 NULL},
     {0x2E,   0x2E,   "0x2E",            "DualNamePrefix",       "NameObject",   "NameSeg NameSeg",                              NULL,
                                                                 "DualNamePath := DualNamePrefix NameSeg NameSeg"},
@@ -184,7 +223,7 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
     {0x5B1F, 0x5B1F, "0x5B1F",          "LoadTableOp",          "TermObject",   "TermArg TermArg TermArg TermArg TermArg TermArg",      NULL,
                                                                 "DefLoadTable := LoadTableOp TermArg TermArg TermArg TermArg TermArg TermArg"},
     {0x5B20, 0x5B20, "0x5B20",          "LoadOp",               "TermObject",   "NameString SuperName",                         NULL,
-                                                                "DefLoad := LoadOp NameString DDBHandleObject"},
+                                                                "DefLoad := LoadOp NameString DdbHandleObject"},
     {0x5B21, 0x5B21, "0x5B21",          "StallOp",              "TermObject",   "TermArg",                                      NULL,
                                                                 "DefStall := StallOp UsecTime"},
     {0x5B22, 0x5B22, "0x5B22",          "SleepOp",              "TermObject",   "TermArg",                                      NULL,
@@ -199,12 +238,12 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
                                                                 "DefReset := ResetOp EventObject"},
     {0x5B27, 0x5B27, "0x5B27",          "ReleaseOp",            "TermObject",   "SuperName",                                    NULL,
                                                                 "DefRelease := ReleaseOp MutexObject"},
-    {0x5B28, 0x5B28, "0x5B28",          "FromBCDOp",            "TermObject",   "TermArg Target",                               NULL,
-                                                                "DefFromBCD := FromBCDOp BCDValue Target"},
-    {0x5B29, 0x5B29, "0x5B29",          "ToBCD",                "TermObject",   "TermArg Target",                               NULL,
-                                                                "DefToBCD := ToBCDOp Operand Target"},
+    {0x5B28, 0x5B28, "0x5B28",          "FromBcdOp",            "TermObject",   "TermArg Target",                               NULL,
+                                                                "DefFromBcd := FromBcdOp BcdValue Target"},
+    {0x5B29, 0x5B29, "0x5B29",          "ToBcd",                "TermObject",   "TermArg Target",                               NULL,
+                                                                "DefToBcd := ToBcdOp Operand Target"},
     {0x5B2A, 0x5B2A, "0x5B2A",          "UnloadOp",             "TermObject",   "SuperName",                                    NULL,
-                                                                "DefUnload := UnloadOp DDBHandleObject"},
+                                                                "DefUnload := UnloadOp DdbHandleObject"},
     {0x5B30, 0x5B30, "0x5B30",          "RevisionOp",           "DataObject",   NULL,                                           NULL,
                                                                 NULL},
     {0x5B31, 0x5B31, "0x5B31",          "DebugOp",              "DebugObject",  NULL,                                           NULL,
@@ -213,18 +252,18 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
                                                                 "DefFatal := FatalOp FatalType FatalCode FatalArg"},
     {0x5B33, 0x5B33, "0x5B33",          "TimerOp",              "TermObject",   NULL,                                           NULL,
                                                                 "DefTimer := TimerOp"},
-    {0x5B80, 0x5B80, "0x5B80",          "OpRegionOp",           "TermObject",   "NameString ByteData TermArg TermArg",          NULL,
+    {0x5B80, 0x5B80, "0x5B80",          "OperationRegionOp",    "TermObject",   "NameString ByteData TermArg TermArg",          NULL,
                                                                 "DefOpRegion := OpRegionOp NameString RegionSpace RegionOffset RegionLen"},
     {0x5B81, 0x5B81, "0x5B81",          "FieldOp",              "TermObject",   "NameString ByteData",                          "FieldList",
                                                                 "DefField := FieldOp PkgLength NameString FieldFlags FieldList"},
-    {0x5B82, 0x5B82, "0x5B82",          "DeviceOp",             "TermObject",   "NameString",                                   "ObjectList",
-                                                                "DefDevice := DeviceOp PkgLength NameString ObjectList"},
-    {0x5B83, 0x5B83, "0x5B83",          "ProcessorOp",          "TermObject",   "NameString ByteData DWordData ByteData",       "ObjectList",
-                                                                "DefProcessor := ProcessorOp PkgLength NameString ProcID PblkAddr PblkLen ObjectList"},
-    {0x5B84, 0x5B84, "0x5B84",          "PowerResOp",           "TermObject",   "NameString ByteData WordData",                 "ObjectList",
-                                                                "DefPowerRes := PowerResOp PkgLength NameString SystemLevel ResourceOrder ObjectList"},
-    {0x5B85, 0x5B85, "0x5B85",          "ThermalZoneOp",        "TermObject",   "NameString",                                   "ObjectList",
-                                                                "DefThermalZone := ThermalZoneOp PkgLength NameString ObjectList"},
+    {0x5B82, 0x5B82, "0x5B82",          "DeviceOp",             "TermObject",   "NameString",                                   "TermList",
+                                                                "DefDevice := DeviceOp PkgLength NameString TermList"},
+    {0x5B83, 0x5B83, "0x5B83",          "ProcessorOp",          "TermObject",   "NameString ByteData DWordData ByteData",       "TermList",
+                                                                "DefProcessor := ProcessorOp PkgLength NameString ProcId PblkAddr PblkLen TermList"},
+    {0x5B84, 0x5B84, "0x5B84",          "PowerResOp",           "TermObject",   "NameString ByteData WordData",                 "TermList",
+                                                                "DefPowerRes := PowerResOp PkgLength NameString SystemLevel ResourceOrder TermList"},
+    {0x5B85, 0x5B85, "0x5B85",          "ThermalZoneOp",        "TermObject",   "NameString",                                   "TermList",
+                                                                "DefThermalZone := ThermalZoneOp PkgLength NameString TermList"},
     {0x5B86, 0x5B86, "0x5B86",          "IndexFieldOp",         "TermObject",   "NameString NameString ByteData",               "FieldList",
                                                                 "DefIndexField := IndexFieldOp PkgLength NameString NameString FieldFlags FieldList"},
     {0x5B87, 0x5B87, "0x5B87",          "BankFieldOp",          "TermObject",   "NameString NameString TermArg ByteData",       "FieldList",
@@ -298,13 +337,13 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
     {0x7B,   0x7B,   "0x7B",            "AndOp",                "TermObject",   "TermArg TermArg Target",                       NULL,
                                                                 "DefAnd := AndOp Operand Operand Target"},
     {0x7C,   0x7C,   "0x7C",            "NandOp",               "TermObject",   "TermArg TermArg Target",                       NULL,
-                                                                "DefNAnd := NandOp Operand Operand Target"},
+                                                                "DefNand := NandOp Operand Operand Target"},
     {0x7D,   0x7D,   "0x7D",            "OrOp",                 "TermObject",   "TermArg TermArg Target",                       NULL,
                                                                 "DefOr := OrOp Operand Operand Target"},
     {0x7E,   0x7E,   "0x7E",            "NorOp",                "TermObject",   "TermArg TermArg Target",                       NULL,
-                                                                "DefNOr := NorOp Operand Operand Target"},
+                                                                "DefNor := NorOp Operand Operand Target"},
     {0x7F,   0x7F,   "0x7F",            "XorOp",                "TermObject",   "TermArg TermArg Target",                       NULL,
-                                                                "DefXOr := XorOp Operand Operand Target"},
+                                                                "DefXor := XorOp Operand Operand Target"},
     {0x80,   0x80,   "0x80",            "NotOp",                "TermObject",   "TermArg Target",                               NULL,
                                                                 "DefNot := NotOp Operand Target"},
     {0x81,   0x81,   "0x81",            "FindSetLeftBitOp",     "TermObject",   "TermArg Target",                               NULL,
@@ -338,23 +377,23 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
     {0x8F,   0x8F,   "0x8F",            "CreateQWordFieldOp",   "TermObject",   "TermArg TermArg NameString",                   NULL,
                                                                 "DefCreateQWordField := CreateQWordFieldOp SourceBuff ByteIndex NameString"},
     {0x90,   0x90,   "0x90",            "LAndOp",               "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLAnd := LandOp Operand Operand"},
+                                                                "DefLAnd := LAndOp Operand Operand"},
     {0x91,   0x91,   "0x91",            "LOrOp",                "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLOr := LorOp Operand Operand"},
+                                                                "DefLOr := LOrOp Operand Operand"},
     {0x92,   0x92,   "0x92",            "LNotOp",               "TermObject",   "TermArg",                                      NULL,
-                                                                "DefLNot := LnotOp Operand"},
+                                                                "DefLNot := LNotOp Operand"},
     {0x9293, 0x9293, "0x9293",          "LNotEqualOp",          "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLNotEqual := LnotEqualOp Operand Operand"},
+                                                                "DefLNotEqual := LNotEqualOp Operand Operand"},
     {0x9294, 0x9294, "0x9294",          "LLessEqualOp",         "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLLessEqual := LlessEqualOp Operand Operand"},
+                                                                "DefLLessEqual := LLessEqualOp Operand Operand"},
     {0x9295, 0x9295, "0x9295",          "LGreaterEqualOp",      "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLGreaterEqual := LgreaterEqualOp Operand Operand"},
+                                                                "DefLGreaterEqual := LGreaterEqualOp Operand Operand"},
     {0x93,   0x93,   "0x93",            "LEqualOp",             "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLEqual := LequalOp Operand Operand"},
+                                                                "DefLEqual := LEqualOp Operand Operand"},
     {0x94,   0x94,   "0x94",            "LGreaterOp",           "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLGreater := LgreaterOp Operand Operand"},
+                                                                "DefLGreater := LGreaterOp Operand Operand"},
     {0x95,   0x95,   "0x95",            "LLessOp",              "TermObject",   "TermArg TermArg",                              NULL,
-                                                                "DefLLess := LlessOp Operand Operand"},
+                                                                "DefLLess := LLessOp Operand Operand"},
     {0x96,   0x96,   "0x96",            "ToBufferOp",           "TermObject",   "TermArg Target",                               NULL,
                                                                 "DefToBuffer := ToBufferOp Operand Target"},
     {0x97,   0x97,   "0x97",            "ToDecimalStringOp",    "TermObject",   "TermArg Target",                               NULL,
@@ -379,8 +418,8 @@ const AH_AML_OPCODE         AmlOpcodeInfo[] =
                                                                 "DefElse := Nothing | <ElseOp PkgLength TermList>"},
     {0xA2,   0xA2,   "0xA2",            "WhileOp",              "TermObject",   "TermArg",                                      "TermList",
                                                                 "DefWhile := WhileOp PkgLength Predicate TermList"},
-    {0xA3,   0xA3,   "0xA3",            "NoopOp",               "TermObject",   NULL,                                           NULL,
-                                                                "DefNoop := NoopOp"},
+    {0xA3,   0xA3,   "0xA3",            "NoOpOp",               "TermObject",   NULL,                                           NULL,
+                                                                "DefNoOp := NoOpOp"},
     {0xA4,   0xA4,   "0xA4",            "ReturnOp",             "TermObject",   "TermArg",                                      NULL,
                                                                 "DefReturn := ReturnOp ArgObject"},
     {0xA5,   0xA5,   "0xA5",            "BreakOp",              "TermObject",   NULL,                                           NULL,
