@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2019, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -111,19 +111,48 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
-
 #include "acpisrc.h"
-#include "acapps.h"
 
 /* Local prototypes */
 
-int
-AsStricmp (
-    char                    *String1,
-    char                    *String2);
-
 int
 AsExaminePaths (
     ACPI_CONVERSION_TABLE   *ConversionTable,
@@ -159,6 +188,7 @@ struct stat             Gbl_StatBuf;
 char                    *Gbl_FileBuffer;
 UINT32                  Gbl_FileSize;
 UINT32                  Gbl_FileType;
+BOOLEAN                 Gbl_CheckAscii = FALSE;
 BOOLEAN                 Gbl_VerboseMode = FALSE;
 BOOLEAN                 Gbl_QuietMode = FALSE;
 BOOLEAN                 Gbl_BatchMode = FALSE;
@@ -172,39 +202,7 @@ BOOLEAN                 Gbl_Cleanup = FALSE;
 BOOLEAN                 Gbl_IgnoreTranslationEscapes = FALSE;
 
 #define AS_UTILITY_NAME             "ACPI Source Code Conversion Utility"
-#define AS_SUPPORTED_OPTIONS        "cdhilqsuv^y"
-
-
-/******************************************************************************
- *
- * FUNCTION:    AsStricmp
- *
- * DESCRIPTION: Implementation of the non-ANSI stricmp function (compare
- *              strings with no case sensitivity)
- *
- ******************************************************************************/
-
-int
-AsStricmp (
-    char                    *String1,
-    char                    *String2)
-{
-    int                     c1;
-    int                     c2;
-
-
-    do
-    {
-        c1 = tolower ((int) *String1);
-        c2 = tolower ((int) *String2);
-
-        String1++;
-        String2++;
-    }
-    while ((c1 == c2) && (c1));
-
-    return (c1 - c2);
-}
+#define AS_SUPPORTED_OPTIONS        "acdhilqsuv^y"
 
 
 /******************************************************************************
@@ -250,7 +248,7 @@ AsExaminePaths (
         return (0);
     }
 
-    if (!AsStricmp (Source, Target))
+    if (!AcpiUtStricmp (Source, Target))
     {
         printf ("Target path is the same as the source path, overwrite?\n");
         Response = getchar ();
@@ -314,8 +312,8 @@ AsDisplayStats (
     printf ("%8u Total bytes (%.1fK/file)\n",
         Gbl_TotalSize, ((double) Gbl_TotalSize/Gbl_Files)/1024);
     printf ("%8u Tabs found\n", Gbl_Tabs);
-    printf ("%8u Missing if/else braces\n", Gbl_MissingBraces);
-    printf ("%8u Non-ANSI comments found\n", Gbl_NonAnsiComments);
+    printf ("%8u Missing if/else/while braces\n", Gbl_MissingBraces);
+    printf ("%8u Non-ANSI // comments found\n", Gbl_NonAnsiComments);
     printf ("%8u Total Lines\n", Gbl_TotalLines);
     printf ("%8u Lines of code\n", Gbl_SourceLines);
     printf ("%8u Lines of non-comment whitespace\n", Gbl_WhiteLines);
@@ -331,7 +329,8 @@ AsDisplayStats (
     if ((Gbl_CommentLines + Gbl_NonAnsiComments) > 0)
     {
         printf ("%8.1f Ratio of code to comments\n",
-            ((float) Gbl_SourceLines / (float) (Gbl_CommentLines + Gbl_NonAnsiComments)));
+            ((float) Gbl_SourceLines /
+            (float) (Gbl_CommentLines + Gbl_NonAnsiComments)));
     }
 
     if (!Gbl_TotalLines)
@@ -363,6 +362,7 @@ AsDisplayUsage (
 
     ACPI_USAGE_HEADER ("acpisrc [-c|l|u] [-dsvy] <SourceDir> <DestinationDir>");
 
+    ACPI_OPTION ("-a <file>",   "Check entire file for non-printable characters");
     ACPI_OPTION ("-c",          "Generate cleaned version of the source");
     ACPI_OPTION ("-h",          "Insert dual-license header into all modules");
     ACPI_OPTION ("-i",          "Cleanup macro indentation");
@@ -374,6 +374,7 @@ AsDisplayUsage (
     ACPI_OPTION ("-s",          "Generate source statistics only");
     ACPI_OPTION ("-v",          "Display version information");
     ACPI_OPTION ("-vb",         "Verbose mode");
+    ACPI_OPTION ("-vd",         "Display build date and time");
     ACPI_OPTION ("-y",          "Suppress file overwrite prompts");
 }
 
@@ -478,6 +479,11 @@ main (
             Gbl_VerboseMode = TRUE;
             break;
 
+        case 'd':
+
+            printf (ACPI_COMMON_BUILD_TIME);
+            return (0);
+
         default:
 
             printf ("Unknown option: -v%s\n", AcpiGbl_Optarg);
@@ -507,6 +513,11 @@ main (
         Gbl_QuietMode = TRUE;
         break;
 
+    case 'a':
+
+        Gbl_CheckAscii = TRUE;
+        break;
+
     default:
 
         AsDisplayUsage ();
@@ -522,6 +533,14 @@ main (
         return (-1);
     }
 
+    /* This option checks the entire file for printable ascii chars */
+
+    if (Gbl_CheckAscii)
+    {
+        AsProcessOneFile (NULL, NULL, NULL, 0, SourcePath, FILE_TYPE_SOURCE);
+        return (0);
+    }
+
     TargetPath = argv[AcpiGbl_Optind+1];
 
     if (!ConversionTable)
@@ -560,21 +579,31 @@ main (
     }
     else
     {
+        if (Gbl_CheckAscii)
+        {
+            AsProcessOneFile (NULL, NULL, NULL, 0,
+                SourcePath, FILE_TYPE_SOURCE);
+            return (0);
+        }
+
         /* Process a single file */
 
         /* Differentiate between source and header files */
 
         if (strstr (SourcePath, ".h"))
         {
-            AsProcessOneFile (ConversionTable, NULL, TargetPath, 0, SourcePath, FILE_TYPE_HEADER);
+            AsProcessOneFile (ConversionTable, NULL, TargetPath, 0,
+                SourcePath, FILE_TYPE_HEADER);
         }
         else if (strstr (SourcePath, ".c"))
         {
-            AsProcessOneFile (ConversionTable, NULL, TargetPath, 0, SourcePath, FILE_TYPE_SOURCE);
+            AsProcessOneFile (ConversionTable, NULL, TargetPath, 0,
+                SourcePath, FILE_TYPE_SOURCE);
         }
         else if (strstr (SourcePath, ".patch"))
         {
-            AsProcessOneFile (ConversionTable, NULL, TargetPath, 0, SourcePath, FILE_TYPE_PATCH);
+            AsProcessOneFile (ConversionTable, NULL, TargetPath, 0,
+                SourcePath, FILE_TYPE_PATCH);
         }
         else
         {
@@ -585,6 +614,5 @@ main (
     /* Always display final summary and stats */
 
     AsDisplayStats ();
-
     return (0);
 }
